// Copyright 2019 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Package event provides a set of packages that cover the main
// concepts of telemetry in an implementation agnostic way.
package event
