// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Package gce defines logic specific to verification of GCE-specific attestations.
package gce

// FirmwareCertGUID is the extended report GUID table GUID for a firmware certificate on GCE.
const FirmwareCertGUID = "9f4116cd-c503-4f5a-8f6f-fb68882f4ce2"
